@@ -10,7 +10,9 @@ use std::time::Duration;
 use crate::std::sync::{Mutex, MutexGuard};
 use std::marker::PhantomData;
 
-use std::collections::{HashMap as Map, hash_map::IntoIter as IntoIter, hash_map::Iter as MapIter, hash_map::IterMut as MapIterMut};
+use std::collections::{HashMap as Map, hash_map::IntoIter as IntoIter, hash_map::Iter as MapIter, hash_map::IterMut as MapIterMut, HashMap};
+use serde::ser::SerializeMap;
+use serde::{Deserializer, Serialize, Serializer};
 
 
 pub type SyncHashMap<K, V> = SyncMapImpl<K, V>;
@@ -139,6 +141,25 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
         }
     }
 
+    pub fn from(map: Map<K, V>) -> Self where K: Clone + Eq + Hash {
+        let s = Self::with_capacity(map.capacity());
+        match s.dirty.lock() {
+            Ok(mut m) => {
+                unsafe {
+                    (&mut *s.read.get()).clear();
+                }
+                *m = map;
+                unsafe {
+                    for (k, v) in m.iter() {
+                        (&mut *s.read.get()).insert(k.clone(), v);
+                    }
+                }
+            }
+            Err(_) => {}
+        }
+        s
+    }
+
 
     /// Returns a reference to the value corresponding to the key.
     ///
@@ -159,7 +180,7 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
     /// assert_eq!(*map.get(&1).unwrap(), "a");
     /// assert_eq!(map.get(&2).is_none(), true);
     /// ```
-    pub fn get<Q: ?Sized>(&self, k: &Q) -> Option<SyncMapRef<'_, V>>
+    pub fn get<Q: ?Sized>(&self, k: &Q) -> Option<&V>
         where
             K: Borrow<Q>,
             Q: Hash + Eq,
@@ -172,9 +193,7 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
                     if s.is_null() {
                         return None;
                     }
-                    Some(SyncMapRef {
-                        value: Some(&**s)
-                    })
+                    Some(&**s)
                 }
             }
         }
@@ -251,34 +270,6 @@ pub unsafe fn change_lifetime_mut<'a, 'b, T>(x: &'a mut T) -> &'b mut T {
 }
 
 
-pub struct SyncMapRef<'a, V> {
-    value: Option<&'a V>,
-}
-
-impl<V> Deref for SyncMapRef<'_, V> {
-    type Target = V;
-
-    fn deref(&self) -> &Self::Target {
-        self.value.as_ref().unwrap()
-    }
-}
-
-impl<V> Debug for SyncMapRef<'_, V> where V: Debug {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        self.value.unwrap().fmt(f)
-    }
-}
-
-
-impl<V> PartialEq<Self> for SyncMapRef<'_, V> where V: Eq {
-    fn eq(&self, other: &Self) -> bool {
-        self.value.eq(&other.value)
-    }
-}
-
-impl<V> Eq for SyncMapRef<'_, V> where V: Eq {}
-
-
 pub struct SyncMapRefMut<'a, K, V> {
     g: MutexGuard<'a, Map<K, V>>,
     value: Option<&'a mut V>,
@@ -397,6 +388,31 @@ impl<K, V> IntoIterator for SyncMapImpl<K, V> where
 }
 
 
+impl <K,V>From<Map<K,V>> for SyncMapImpl<K,V> {
+    fn from(arg: Map<K, V>) -> Self {
+        Self::from(arg)
+    }
+}
+
+impl<K, V> serde::Serialize for SyncMapImpl<K, V> where K: Eq + Hash + Clone + Serialize, V: Serialize {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error> where S: Serializer {
+        let mut m = serializer.serialize_map(Some(self.len()))?;
+        for (k, v) in self.iter() {
+            m.serialize_key(k)?;
+            m.serialize_value(v)?;
+        }
+        m.end()
+    }
+}
+
+impl<'de, K, V> serde::Deserialize<'de> for SyncMapImpl<K, V> where K: Eq + Hash + Clone + serde::Deserialize<'de>, V: serde::Deserialize<'de> {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error> where D: Deserializer<'de> {
+        let m = Map::deserialize(deserializer)?;
+        Ok(Self::from(m))
+    }
+}
+
+
 #[cfg(test)]
 mod test {
     use std::collections::HashMap;
@@ -487,7 +503,7 @@ mod test {
         let m = SyncHashMap::<i32, i32>::new();
         let insert = m.insert(1, 2);
         let g = m.get(&1).unwrap();
-        assert_eq!(2, *g.deref());
+        assert_eq!(&2, g);
     }
 
     #[derive(Clone, Debug, Eq, PartialEq, Hash)]
@@ -513,7 +529,7 @@ mod test {
         assert_eq!(true, m.is_empty());
         assert_eq!(true, m.dirty.lock().unwrap().is_empty());
         assert_eq!(None, m.get(&1));
-        assert_eq!(A { inner: 0 }, *g.deref());
+        assert_eq!(&A { inner: 0 }, g);
     }
 
     #[test]
