@@ -8,6 +8,7 @@ mod semphore;
 mod sync_flag;
 mod wait_group;
 mod sync_map;
+mod once;
 
 pub(crate) mod atomic_dur;
 #[cfg(not(unix))]
@@ -26,3 +27,4 @@ pub use self::semphore::Semphore;
 pub use self::sync_flag::SyncFlag;
 pub use self::wait_group::*;
 pub use self::sync_map::*;
+pub use self::once::*;
